      subroutine fladv(q,qbc,dz,sw,deriv,ndep,qf,dqfdq,dqfdz)
c
c  Van Leer (J. Comp. Phys. 14, 361-370) second order upwind 
c  advection scheme.
c
c  The zones are numbered from        1 to ndep-1 while the 
c      boundaries are numbered from   1 to ndep.
c      It is assumed that the ghost zones are of the same size 
c      as the first and last zones respectively.
c
c  Inputs:
c  q    (ndep)   quantity being advected 
c  qbc  (4)      boundary condition for a for two zones to the 
c                "left" of the first zone (1 and 2) and to the 
c                "right" of the last zone (3 and 4) 
c  dz   (ndep)   distance between zone boundaries
c  sw   (ndep)   = 0.5 if urel=u*dt-z > 0
c                =-0.5 if urel        < 0
c  deriv         =.true. if derivatives are to be calculated
c  Output:
c  qf   (ndep)         Van Leer average of quantity being advected
c  dqfdq(ndep,-2:2)    D[qf]/D[q(k-2,k+2)]
c  dqfdz(ndep,-2:3)    D[qf]/D[z(k-2,k+3)]
c
c  Viggo Hansteen         5/1-96                 Ver. 1.0  
c
      use vartype
c      use params
      implicit none
      real(sp), parameter :: one=1.0_sp
      integer(i4b), parameter :: mdep=500
c
      integer(i4b) ndep
      real(sp) q(ndep),dz(ndep),sw(ndep),qbc(4)
      real(sp) qf(ndep),dqfdq(ndep,-2:2),dqfdz(ndep,-2:3)
      logical deriv
c
c  Local variables
c
      real(sp) a(-1:mdep+1),dx(-1:mdep+1),dxh(-1:mdep+1)
      real(sp) da(-1:mdep+1),dela(0:mdep),amid2(0:mdep)
      real(sp) ddadx(0:mdep,-2:3),ddada(0:mdep,-2:2)
      real(sp) hms,hps
      integer k,sw2(0:mdep)
c
c  =============================================================== 
c
      if(ndep > mdep) stop 'fladv: ndep > mdep'
c
c  Apply boundary conditions and move to local variables
c
      k=1
        a(k-2)=qbc(1)
        a(k-1)=qbc(2)
        dx(k-1)=dz(k)
        dx(k-2)=dz(k)
      do k=1,ndep-1
        a(k)=q(k)
        dx(k)=dz(k)
      enddo
      k=ndep
        a(k)=qbc(3)
        a(k+1)=qbc(4)
        dx(k)=dz(k)
        dx(k+1)=dz(k)
c
      do k=0,ndep+1
        dxh(k)=half*(dx(k-1)+dx(k))
        da(k)=(a(k)-a(k-1))/dxh(k)
      enddo
c
c  Advected quantity is given by first order upwind scheme +
c  a second order correction term 
c    d2a[k]    = +("up")-("down")dxh[k]*dela[k]/2  with
c    dela[k]   = 2 da[k]da[k+1]/(da[k]+da[k+1])  ; if da[k]da[k+1] > 0
c              = 0                               ; otherwise 
c    and
c    da[k]= (a[k+1]-a[k])/dxh[k]
c
      do k=0,ndep
        sw2(k)=0
        if(da(k)*da(k+1).gt.zero)sw2(k)=1
      enddo
      if(deriv)then
        do k=0,ndep
          ddadx(k,-1)=half*da(k)/dxh(k)
          ddadx(k,1)=-half*da(k)/dxh(k)
        enddo
      endif
c
      do k=0,ndep
        amid2(k)=(da(k)+da(k+1))*sw2(k)+(1-sw2(k))
        dela(k)=da(k)*da(k+1)*sw2(k)/amid2(k)*dx(k)
      enddo
      if(deriv)then
        do k=0,ndep
          ddada(k,-1)=da(k+1)*sw2(k)/amid2(k)*
     .             (-one+da(k)/amid2(k))*dx(k)/dxh(k) 
          ddada(k,0)=da(k+1)*sw2(k)/amid2(k)*
     .             (one-da(k)/amid2(k))*dx(k)/dxh(k)
     .            +da(k)*sw2(k)/amid2(k)*
     .             (-one+da(k+1)/amid2(k))*dx(k)/dxh(k+1)
          ddada(k,1)=da(k)*sw2(k)/amid2(k)*
     .             (one-da(k+1)/amid2(k))*dx(k)/dxh(k+1)
c
          ddadx(k,-1)=da(k+1)*sw2(k)/amid2(k)*dx(k)*ddadx(k,-1)
     .               -dela(k)/amid2(k)*sw2(k)*ddadx(k,-1)
          ddadx(k,0)=da(k)*sw2(k)/amid2(k)*dx(k)*ddadx(k+1,-1)
     .               -dela(k)/amid2(k)*sw2(k)*ddadx(k+1,-1)
     .               -dela(k)/dx(k)
          ddadx(k,1)=da(k+1)*sw2(k)/amid2(k)*dx(k)*ddadx(k,1)
     .               -dela(k)/amid2(k)*sw2(k)*ddadx(k,1)
     .               +dela(k)/dx(k)
          ddadx(k,2)=da(k)*sw2(k)/amid2(k)*dx(k)*ddadx(k+1,1)
     .               -dela(k)/amid2(k)*sw2(k)*ddadx(k+1,1)
        enddo
      endif
c
      do k=1,ndep
        qf(k)=(half+sw(k))*(a(k-1)+dela(k-1))+
     .        (half-sw(k))*(a(k)-dela(k))
      enddo
      if(deriv)then
        do k=1,ndep
          hps=half+sw(k)
          hms=half-sw(k)
          dqfdz(k,-2)=hps*ddadx(k-1,-1)
          dqfdz(k,-1)=hps*ddadx(k-1,0)-hms*ddadx(k,-1)
          dqfdz(k,0)=hps*ddadx(k-1,1)-hms*ddadx(k,0)
          dqfdz(k,1)=hps*ddadx(k-1,2)-hms*ddadx(k,1)
          dqfdz(k,2)=-hms*ddadx(k,2)
c
          dqfdq(k,-2)=hps*ddada(k-1,-1)
          dqfdq(k,-1)=hps*(one+ddada(k-1,0))-hms*ddada(k,-1)
          dqfdq(k,0)=hps*ddada(k-1,1)+hms*(one-ddada(k,0))
          dqfdq(k,1)=-hms*ddada(k,1)
          dqfdq(k,2)=zero
        enddo
      endif
c
      return
      end

