program implicit
  use vartype
  use work
  implicit none
  integer(i4b) :: ieq,maxit,it,iter,maxl(1),i,l,kb,k,idpmax,ieqmax
  real(sp) :: dzc,dtmin,dtmax,tend,dtsave,uconst,t,errmax,cormax&
       &,cordtm,dtcourant,tsave,cortmp,cordt,courantfac
  logical(lgt) :: conver
  character*50 text,savefile
  real eltime,etime,tarray(2)
  real(sp), dimension(:), pointer :: abse

  data dzc/10.0/,dtmin/1e-2/,tend/1000./,uconst/1./,maxit/5/,errmax/1e-3/&
       &,dtmax/1e2/,dtsave/100/,cordtm/0.9/,savefile/'out'/&
       &,courantfac/0.9/

  namelist /input/ dt,dtmin,dtmax,tend,dtsave,dzc,ngrid,uconst,beta&
       &,maxit,errmax,cordtm,courantfac,savefile


  dt = 0.1
  ngrid = 100
  a = 1.0
  beta = 0.6

  !Meaning of the namelist input parameters (only values that differ
  ! from the default values above need to be specified):

  !DT: Time step at start of run (will change later)
  !DTMIN: Minimum time step
  !DTMAX: Max. time step
  !TEND: Integration time from T=0 to T=TEND
  !DTSAVE: Time step between each time the solution is saved
  !DZC: Altitude grid spacing
  !NGRID: Number of grid points
  !UCONST: The velocity (which is constant for now)
  !BETA: Degree of "implicitness" in the calculation.  BETA=1 means
  ! fully implicit time integration, BETA=0 fully explicit.
  !MAXIT: Max. number of iterations before we cut the time step in
  ! half
  !ERRMAX: Maximum error allowed in the solution
  !CORDTM: Max. allowed relative change in any one variable (for now only
  ! density) in a single time step.
  !COURANTFAC: For (semi-) explicit time integration, the maximum
  ! allowed time step is COURANTFAC*DTCOURANT (where DTCOURANT is an
  ! estimate of the Courant limit).  So COURANTFAC>1 and BETA=0 will
  ! most likely lead to disaster...
  !SAVEFILE: Name of the output file containing the solution.


  read(*,NML=input)             !read input variables from standard input
  write(*,NML=input)            !to standard output

!!$  read(*,ADVANCE='NO') 'Choose number of grid points: ', ngrid

  allocate(z(ngrid),no(ngrid),nn(ngrid),u(ngrid),v(ngrid),dz(ngrid))
  allocate(e(neq,ngrid),w(neq,ngrid,neq,-2:2))
  allocate(dnhdn(ngrid,-2:2),dnhdz(ngrid,-2:3),dfldn(ngrid,-2:2)) !dnhdz is dummy for now
  allocate(scratch(ngrid,mscratch)) !array for temporary storage

  !write unformatted (use e.g. IDL to read it).  STATUS=NEW means the
  ! file must not already exist.
  open(nunit,FILE=savefile,FORM='UNFORMATTED',STATUS='NEW')

  z(1) = 0.
  do i=2,ngrid
     z(i) = z(i-1)+dzc
  enddo
  dz = eoshift(z,1,z(ngrid)+dzc)-z !in case we have non-constant grid spacing
  v = a*dz                      !volume elements A*DZ

  u = uconst                    !constant speed for now

  ! For now we're just advecting a step function at constant speed uconst:
  no = 0.1
  no(10:20) = 0.2              !create step function
  nn = no

  if (beta<0 .or. beta>1) stop 'beta is outside valid range'

  dtcourant = minval(dz/u)      !courant time limit
  write(*,*) 'Courant time step limit: ', dtcourant, ' s'

  t = 0.                        !initialize time
  tsave = t+dtsave
  call save(t)
  it = 0                        !time index
  do while (t < tend)
     it = it+1
     conver = .false.
     iter = 0
     do while (.not. conver)
        iter = iter + 1
        if (iter > maxit) then  !redo with smaller time step
           dt = dt/2.
           if (dt < dtmin) stop 'dt < dtmin'
           iter = 0
           nn = no              !start iteration from the beginning
           write(*,2) dt
2          format('Time step reduced to ',f7.2,' s')
        end if

        e = 0.
        w = 0.

        ieq = 1                     !which equation we solve for (1=continuity)
        call continuity(ieq)

        ! And when the equations for momentum and pressure
        ! (temperature) have been implemented they will probably
        ! appear here:
!!$        ieq = 2
!!$        call momentum(ieq)
!!$        ieq = 3
!!$        call pressure(ieq)

        ! scale W so that elements have comparable values.  Remember to
        ! multiply by the same scale factor after PENTAG has been called!
        ieq = 1
        do l=1,ngrid
           do kb = 2,-2,-1
              k = l-kb
              if (k >= 1 .and. k<=ngrid) w(:,k,ieq,kb) = w(:,k,ieq,kb)*no(l)
           end do
        end do

        ! And here is a similar scaling of the momentum eqn. derivatives when
        ! they have been implemented.  CSO is perhaps the sound speed.
!!$        ieq = 2
!!$        do l=1,ngrid
!!$           do kb = 2,-2,-1
!!$              k = l-kb
!!$              if (k >= 1 .and. k<=ngrid) w(:,k,ieq,kb) = w(:,k,ieq,kb)*cso(l)
!!$           end do
!!$        end do

       call pentag(neq,ngrid,1,e,w)

        ieq = 1
        !multiply by same factor as when scaling W:
        nn = nn + e(ieq,:)*no         !the new trial solution

!!$        ieq = 2
!!$        un = un + e(ieq,:)*cso

        cormax = 0.
        abse => scratch(:,1)
        do ieq = 1,neq
           abse = abs(e(ieq,:))
           cortmp = maxval(abse)      !maximum error
           if (cormax <= cortmp) then
              maxl = maxloc(abse)
              idpmax = maxl(1)     !grid point where maximum occurs
              ieqmax = ieq            !equation containing max. error
              cormax = cortmp
           endif
        end do

        if (cormax <= errmax) conver = .true. !iteration has converged
     end do
     t = t + dt                 !step time if converged
     no = nn                    !update solution if converged

     if (t >= tsave) then
        call save(t)  !write solution
        write(*,3) t
3       format('Saved at t=',f7.2)
        tsave = t + dtsave
     end if
     write(*,1) t,dt,iter
1    format('t=',f7.2,', dt=',f7.2,', iter=',i4)
     !cordt is the tentative new time step
     if (iter >= maxit-1) then
        ! convergence is slow so reduce time step a bit
        cordt = 0.9*dt
     else
        ! increase time step
        cordt = min(1.4*dt,cordtm/cormax*dt*0.5)
     endif
     ! but if we run explicitly (beta<0.5) impose courant limit:
     if (beta < 0.5) then
        dt = min(cordt,courantfac*dtcourant)
     else
        dt = min(cordt,dtmax)
     endif
     if (t+dt > tsave) dt = tsave-t

  end do                        !t<tend

  close(nunit)

  eltime = etime(tarray)
  write(*,4)  eltime, tarray(1)
4 format('Done.  Elapsed time: ',f7.2,', CPU-time: ',f7.2,' s')

end program implicit
